@@ -3,6 +3,7 @@
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
+    Copyright 2009-2010, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of version 2 of the GNU General Public
@@ -17,19 +18,20 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of the ZynAddSubFX original, modified October 2009
+    This file is a derivative of a ZynAddSubFX original, modified October 2010
 */
 
 #ifndef FILTER_H
 #define FILTER_H
 
+#include "Misc/MiscFuncs.h"
 #include "DSP/Filter_.h"
 #include "DSP/AnalogFilter.h"
 #include "DSP/FormantFilter.h"
 #include "DSP/SVFilter.h"
 #include "Params/FilterParams.h"
 
-class Filter
+class Filter : private MiscFuncs
 {
     public:
         Filter(FilterParams *pars);
