@@ -3,7 +3,7 @@
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
-    Copyright 2009, Alan Calvert
+    Copyright 2009-2010, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of version 2 of the GNU General Public
@@ -18,17 +18,17 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of the ZynAddSubFX original, modified October 2009
+    This file is a derivative of a ZynAddSubFX original, modified October 2010
 */
 
 #ifndef MICROTONAL_H
 #define MICROTONAL_H
 
-#include <cstring>
+#include <string>
 
 using namespace std;
 
-#include "Misc/XMLwrapper.h"
+class XMLwrapper;
 
 #define MAX_OCTAVE_SIZE 128
 
@@ -44,16 +44,16 @@ class Microtonal
         unsigned char Pinvertupdown;
         unsigned char Pinvertupdowncenter;
         unsigned char Penabled;
-        unsigned char PAnote;
-        unsigned char Pscaleshift;
-        float PAfreq;
+        int           PAnote;
+        int           Pscaleshift;
+        float         PAfreq;
 
         // first and last key (to retune)
         unsigned char Pfirstkey;
         unsigned char Plastkey;
 
         // The middle note where scale degree 0 is mapped to
-        unsigned char Pmiddlenote;
+        int           Pmiddlenote;
 
         // Map size
         unsigned char Pmapsize;
@@ -85,7 +85,7 @@ class Microtonal
         int loadline(FILE *file, char *line);
             // loads a line from the text file,
             // ignoring the lines beggining with "!"
-        unsigned char octavesize;
+        int octavesize;
         struct {
             unsigned char type; // 1 for cents or 2 for division
             // the real tuning (eg. +1.05946 for one halftone)
@@ -100,4 +100,9 @@ class Microtonal
       int oscilsize;
 };
 
+inline unsigned char Microtonal::getoctavesize(void)
+{
+    return ((Penabled) ? octavesize : 12);
+}
+
 #endif
