@@ -21,43 +21,40 @@
 #ifndef MUSIC_IO_H
 #define MUSIC_IO_H
 
-#include <pthread.h>
-
-#include "Misc/Master.h"
+#include "Misc/MiscFuncs.h"
 #include "MusicIO/WavRecord.h"
 
-class MusicIO
+class MusicIO : protected MiscFuncs
 {
     public:
         MusicIO();
-        ~MusicIO();
-
+        ~MusicIO() { }
         virtual unsigned int getSamplerate(void) = 0;
         virtual int getBuffersize(void) = 0;
         virtual bool Start(void) = 0;
         virtual void Close(void);
-        int grossLatency(void) { return audioLatency + midiLatency; };
+        bool jacksessionReply(string cmdline) { return false; }
+        int grossLatency(void) { return audioLatency + midiLatency; }
 
     protected:
         bool prepBuffers(bool with_interleaved);
         bool prepRecord(void);
         void getAudio(void);
         void InterleaveShorts(void);
-
         bool setThreadAttributes(pthread_attr_t *attr, bool schedfifo, bool midi = false);
-
         int getMidiController(unsigned char b);
         void setMidiController(unsigned char ch, unsigned int ctrl, int param);
         void setMidiNote(unsigned char chan, unsigned char note);
         void setMidiNote(unsigned char chan, unsigned char note, unsigned char velocity);
-        jsample_t *zynLeft;
-        jsample_t *zynRight;
+
+        float *zynLeft;
+        float *zynRight;
         short int *interleavedShorts;
 
         WavRecord *wavRecorder;
         int rtprio;
-        jack_nframes_t audioLatency; // frames
-        jack_nframes_t midiLatency;  // ""
+        unsigned int audioLatency; // frames
+        unsigned int midiLatency;  // ""
 };
 
 #endif
