@@ -3,7 +3,7 @@
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
-    Copyright 2009, Alan Calvert
+    Copyright 2009-2010, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of version 2 of the GNU General Public
@@ -18,13 +18,19 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of the ZynAddSubFX original, modified October 2009
+    This file is a derivative of a ZynAddSubFX original, modified October 2010
 */
 
 #include <cmath>
 
-#include "Misc/Util.h"
-#include "Misc/Master.h"
+#include "Misc/XMLwrapper.h"
+#include "DSP/FFTwrapper.h"
+#include "Synth/OscilGen.h"
+#include "Synth/Resonance.h"
+#include "Params/EnvelopeParams.h"
+#include "Params/LFOParams.h"
+#include "Params/FilterParams.h"
+#include "Misc/SynthEngine.h"
 #include "Params/PADnoteParameters.h"
 
 PADnoteParameters::PADnoteParameters(FFTwrapper *fft_) : Presets()
@@ -58,16 +64,15 @@ PADnoteParameters::PADnoteParameters(FFTwrapper *fft_) : Presets()
 PADnoteParameters::~PADnoteParameters()
 {
     deletesamples();
-    delete(oscilgen);
-    delete(resonance);
-    delete(FreqEnvelope);
-    delete(FreqLfo);
-    delete(AmpEnvelope);
-    delete(AmpLfo);
-    delete(GlobalFilter);
-    delete(FilterEnvelope);
-    delete(FilterLfo);
-
+    delete oscilgen;
+    delete resonance;
+    delete FreqEnvelope;
+    delete FreqLfo;
+    delete AmpEnvelope;
+    delete AmpLfo;
+    delete GlobalFilter;
+    delete FilterEnvelope;
+    delete FilterLfo;
 }
 
 void PADnoteParameters::defaults(void)
@@ -142,7 +147,7 @@ void PADnoteParameters::deletesample(int n)
         sample[n].smp = NULL;
     }
     sample[n].size = 0;
-    sample[n].basefreq = 440.0;
+    sample[n].basefreq = 440.0f;
 }
 
 void PADnoteParameters::deletesamples(void)
@@ -155,33 +160,33 @@ void PADnoteParameters::deletesamples(void)
 float PADnoteParameters::getprofile(float *smp, int size)
 {
     for (int i = 0; i < size; ++i)
-        smp[i] = 0.0;
+        smp[i] = 0.0f;
 
     const int supersample = 16;
-    float basepar = powf(2.0f, ((1.0f - Php.base.par1 / 127.0f) * 12.0));
-    float freqmult = floorf(powf(2.0f, (Php.freqmult / 127.0f * 5.0f)) + 0.000001);
+    float basepar = powf(2.0f, ((1.0f - Php.base.par1 / 127.0f) * 12.0f));
+    float freqmult = floorf(powf(2.0f, (Php.freqmult / 127.0f * 5.0f)) + 0.000001f);
 
-    float modfreq = floorf(powf(2.0f, (Php.modulator.freq / 127.0f * 5.0f)) + 0.000001);
+    float modfreq = floorf(powf(2.0f, (Php.modulator.freq / 127.0f * 5.0f)) + 0.000001f);
     float modpar1 = powf((Php.modulator.par1 / 127.0f), 4.0f) * 5.0 / sqrtf(modfreq);
-    float amppar1 = powf(2.0, powf((Php.amp.par1 / 127.0f), 2.0f) * 10.0f) - 0.999;
-    float amppar2 = (1.0 - Php.amp.par2 / 127.0) * 0.998 + 0.001;
+    float amppar1 = powf(2.0f, powf((Php.amp.par1 / 127.0f), 2.0f) * 10.0f) - 0.999f;
+    float amppar2 = (1.0f - Php.amp.par2 / 127.0f) * 0.998f + 0.001f;
     float width = powf((150.0f / (Php.width + 22.0f)), 2.0f);
 
     for (int i = 0; i < size * supersample; ++i)
     {
         bool makezero = false;
-        float x = i * 1.0 / (size * (float)supersample);
+        float x = i * 1.0f / (size * (float)supersample);
         float origx = x;
         // do the sizing (width)
-        x = (x - 0.5) * width + 0.5;
-        if (x < 0.0)
+        x = (x - 0.5f) * width + 0.5f;
+        if (x < 0.0f)
         {
-            x = 0.0;
+            x = 0.0f;
             makezero = true;
         } else {
-            if (x >1.0)
+            if (x >1.0f)
             {
-                x = 1.0;
+                x = 1.0f;
                 makezero = true;
             }
         }
@@ -189,10 +194,10 @@ float PADnoteParameters::getprofile(float *smp, int size)
         switch (Php.onehalf)
         {
         case 1:
-            x = x * 0.5 + 0.5;
+            x = x * 0.5f + 0.5f;
             break;
         case 2:
-            x = x * 0.5;
+            x = x * 0.5f;
             break;
         }
 
@@ -200,18 +205,18 @@ float PADnoteParameters::getprofile(float *smp, int size)
         // do the frequency multiplier
         x *= freqmult;
         // do the modulation of the profile
-        x += sinf(x_before_freq_mult * 3.1415926 * modfreq) * modpar1;
-        x = fmodf(x + 1000.0, 1.0) * 2.0 - 1.0;
+        x += sinf(x_before_freq_mult * 3.1415926f * modfreq) * modpar1;
+        x = fmodf(x + 1000.0f, 1.0f) * 2.0f - 1.0f;
         // this is the base function of the profile
         float f;
         switch (Php.base.type)
         {
         case 1:
             f = expf(-(x * x) * basepar);
-            if (f < 0.4)
-                f = 0.0;
+            if (f < 0.4f)
+                f = 0.0f;
             else
-                f = 1.0;
+                f = 1.0f;
             break;
         case 2:
             f = expf(-(fabsf(x)) * sqrtf(basepar));
@@ -221,20 +226,20 @@ float PADnoteParameters::getprofile(float *smp, int size)
             break;
         }
         if (makezero)
-            f = 0.0;
-        float amp = 1.0;
-        origx = origx * 2.0 - 1.0;
+            f = 0.0f;
+        float amp = 1.0f;
+        origx = origx * 2.0f - 1.0f;
         // compute the amplitude multiplier
         switch (Php.amp.type)
         {
         case 1:
-            amp = expf(-(origx * origx) * 10.0 * amppar1);
+            amp = expf(-(origx * origx) * 10.0f * amppar1);
             break;
         case 2:
-            amp = 0.5 * (1.0 + cosf(PI * origx * sqrtf(amppar1 * 4.0 + 1.0)));
+            amp = 0.5f * (1.0f + cosf(PI * origx * sqrtf(amppar1 * 4.0f + 1.0f)));
             break;
         case 3:
-            amp = 1.0 / (powf(origx * (amppar1 * 2.0f + 0.8f), 14.0f) + 1.0);
+            amp = 1.0f / (powf(origx * (amppar1 * 2.0f + 0.8f), 14.0f) + 1.0f);
             break;
         }
         // apply the amplitude multiplier
@@ -244,48 +249,48 @@ float PADnoteParameters::getprofile(float *smp, int size)
             switch (Php.amp.mode)
             {
             case 0:
-                finalsmp = amp * (1.0 - amppar2) + finalsmp * amppar2;
+                finalsmp = amp * (1.0f - amppar2) + finalsmp * amppar2;
                 break;
             case 1:
-                finalsmp *= amp * (1.0 - amppar2) + amppar2;
+                finalsmp *= amp * (1.0f - amppar2) + amppar2;
                 break;
             case 2:
                 finalsmp =
-                    finalsmp / (amp + powf(amppar2, 4.0f) * 20.0 + 0.0001);
+                    finalsmp / (amp + powf(amppar2, 4.0f) * 20.0f + 0.0001f);
                 break;
             case 3:
                 finalsmp =
-                    amp / (finalsmp + powf(amppar2, 4.0f) * 20.0 + 0.0001);
+                    amp / (finalsmp + powf(amppar2, 4.0f) * 20.0f + 0.0001f);
                 break;
             }
         }
         smp[i / supersample] += finalsmp / supersample;
     }
     // normalize the profile (make the max. to be equal to 1.0)
-    float max = 0.0;
+    float max = 0.0f;
     for (int i = 0; i < size; ++i)
     {
         smp[i] = smp[i];
         if (smp[i] > max)
             max = smp[i];
     }
-    if (max < 0.00001)
-        max = 1.0;
+    if (max < 0.00001f)
+        max = 1.0f;
     for (int i = 0; i < size; ++i)
         smp[i] /= max;
 
     if (!Php.autoscale)
-        return 0.5;
+        return 0.5f;
     // compute the estimated perceived bandwidth
-    float sum = 0.0;
+    float sum = 0.0f;
     int i;
     for (i = 0; i < size / 2 - 2; ++i)
     {
         sum += smp[i] * smp[i] + smp[size - i -1] * smp[size - i - 1];
-        if (sum >= 4.0)
+        if (sum >= 4.0f)
             break;
     }
-    float result = 1.0 - 2.0 * i / (float)size;
+    float result = 1.0f - 2.0f * i / (float)size;
     return result;
 }
 
@@ -294,8 +299,8 @@ float PADnoteParameters::getprofile(float *smp, int size)
 float PADnoteParameters::setPbandwidth(int Pbandwidth)
 {
     this->Pbandwidth = Pbandwidth;
-    float result = powf(Pbandwidth / 1000.0, 1.1);
-    result = powf(10.0, result * 4.0) * 0.25;
+    float result = powf(Pbandwidth / 1000.0f, 1.1f);
+    result = powf(10.0f, result * 4.0f) * 0.25f;
     return result;
 }
 
@@ -303,50 +308,50 @@ float PADnoteParameters::setPbandwidth(int Pbandwidth)
 float PADnoteParameters::getNhr(int n)
 {
     float result = 1.0;
-    float par1 = powf(10.0f, -(1.0f - Phrpos.par1 / 255.0f) * 3.0);
-    float par2 = Phrpos.par2 / 255.0;
+    float par1 = powf(10.0f, -(1.0f - Phrpos.par1 / 255.0f) * 3.0f);
+    float par2 = Phrpos.par2 / 255.0f;
 
-    float n0 = n - 1.0;
-    float tmp = 0.0;
+    float n0 = n - 1.0f;
+    float tmp = 0.0f;
     int thresh = 0;
     switch (Phrpos.type)
     {
     case 1:
-        thresh = (int)(par2 * par2 * 100.0) + 1;
+        thresh = lrintf(par2 * par2 * 100.0f) + 1;
         if (n < thresh)
             result = n;
         else
-            result = 1.0 + n0 + (n0 - thresh + 1.0) * par1 * 8.0;
+            result = 1.0f + n0 + (n0 - thresh + 1.0f) * par1 * 8.0f;
         break;
     case 2:
-        thresh = (int)(par2 * par2 * 100.0) + 1;
+        thresh = lrintf(par2 * par2 * 100.0f) + 1;
         if (n < thresh)
             result = n;
         else
-            result = 1.0 + n0 - (n0 - thresh + 1.0) * par1 * 0.90;
+            result = 1.0f + n0 - (n0 - thresh + 1.0f) * par1 * 0.90f;
         break;
     case 3:
-        tmp = par1 * 100.0 + 1.0;
-        result = powf(n0 / tmp, (1.0f - par2 * 0.8f)) * tmp + 1.0;
+        tmp = par1 * 100.0f + 1.0f;
+        result = powf(n0 / tmp, (1.0f - par2 * 0.8f)) * tmp + 1.0f;
         break;
     case 4:
-        result = n0 * (1.0 - par1) + powf(n0 * 0.1f, par2 * 3.0f + 1.0f) * par1 * 10.0 + 1.0;
+        result = n0 * (1.0f - par1) + powf(n0 * 0.1f, par2 * 3.0f + 1.0f) * par1 * 10.0f + 1.0f;
         break;
     case 5:
-        result = n0 + sinf(n0 * par2 * par2 * PI * 0.999) * sqrtf(par1) * 2.0 + 1.0;
+        result = n0 + sinf(n0 * par2 * par2 * PI * 0.999f) * sqrtf(par1) * 2.0f + 1.0f;
         break;
     case 6:
-        tmp = powf((par2 * 2.0), 2.0) + 0.1;
-        result = n0 * powf(1.0f + par1 * powf(n0 * 0.8f, tmp), tmp) + 1.0;
+        tmp = powf((par2 * 2.0f), 2.0f) + 0.1f;
+        result = n0 * powf(1.0f + par1 * powf(n0 * 0.8f, tmp), tmp) + 1.0f;
         break;
     default:
         result=n;
         break;
     }
-    float par3 = Phrpos.par3 / 255.0;
+    float par3 = Phrpos.par3 / 255.0f;
     float iresult = floorf(result + 0.5f);
-    float dresult = result-iresult;
-    result = iresult + (1.0 - par3) * dresult;
+    float dresult = result - iresult;
+    result = iresult + (1.0f - par3) * dresult;
     return result;
 }
 
@@ -363,74 +368,74 @@ void PADnoteParameters::generatespectrum_bandwidthMode(float *spectrum,
     //    spectrum[i] = 0.0;
     memset(spectrum, 0, sizeof(float) * size);
 
-    //float harmonics[zynMaster->getOscilsize() / 2];
-    float harmonics[half_oscilsize];
-    memset(harmonics, 0, half_oscilsize * sizeof(float));
+    //float harmonics[synth->getOscilsize() / 2];
+    float harmonics[synth->halfoscilsize];
+    memset(harmonics, 0, synth->halfoscilsize * sizeof(float));
 
     // get the harmonic structure from the oscillator (I am using the frequency amplitudes, only)
     oscilgen->get(harmonics, basefreq, false);
 
     // normalize
-    float max = 0.0;
-    for (int i = 0; i < half_oscilsize; ++i)
+    float max = 0.0f;
+    for (int i = 0; i < synth->halfoscilsize; ++i)
         if (harmonics[i] > max)
             max = harmonics[i];
-    if (max < 0.000001)
+    if (max < 0.000001f)
         max = 1;
-    for (int i = 0; i < half_oscilsize; ++i)
+    for (int i = 0; i < synth->halfoscilsize; ++i)
         harmonics[i] /= max;
-    for (int nh = 1; nh < half_oscilsize; ++nh)
+    for (int nh = 1; nh < synth->halfoscilsize; ++nh)
     {   //for each harmonic
         float realfreq = getNhr(nh) * basefreq;
-        if (realfreq > samplerate * 0.49999)
+        if (realfreq > synth->samplerate_f * 0.49999f)
             break;
-        if (realfreq < 20.0)
+        if (realfreq < 20.0f)
             break;
-        if (harmonics[nh - 1] < 1e-4)
+        if (harmonics[nh - 1] < 1e-4f)
             continue;
         //compute the bandwidth of each harmonic
         float bandwidthcents = setPbandwidth(Pbandwidth);
         float bw = (powf(2.0f, bandwidthcents / 1200.0f) - 1.0f) * basefreq / bwadjust;
-        float power = 1.0;
+        float power = 1.0f;
         switch (Pbwscale)
         {
         case 0:
-            power = 1.0;
+            power = 1.0f;
             break;
         case 1:
-            power = 0.0;
+            power = 0.0f;
             break;
         case 2:
-            power = 0.25;
+            power = 0.25f;
             break;
         case 3:
-            power = 0.5;
+            power = 0.5f;
             break;
         case 4:
-            power = 0.75;
+            power = 0.75f;
             break;
         case 5:
-            power = 1.5;
+            power = 1.5f;
             break;
         case 6:
-            power = 2.0;
+            power = 2.0f;
             break;
         case 7:
             power = -0.5;
             break;
         }
         bw = bw * powf(realfreq / basefreq, power);
-        int ibw = (int)((bw / (samplerate * 0.5) * size)) + 1;
+        int ibw = lrintf((bw / (synth->samplerate_f * 0.5f) * size)) + 1;
         float amp = harmonics[nh - 1];
         if (resonance->Penabled)
             amp *= resonance->getfreqresponse(realfreq);
         if (ibw > profilesize)
         {   // if the bandwidth is larger than the profilesize
             float rap = sqrtf((float)profilesize / (float)ibw);
-            int cfreq = (int)(realfreq / (samplerate * 0.5) * size) - ibw / 2;
+            int cfreq = lrintf(realfreq / (synth->halfsamplerate_f) * size) - ibw / 2;
             for (int i = 0; i < ibw; ++i)
             {
-                int src = (int)(i * rap * rap);
+                int src = lrintf(i * rap * rap);
                 int spfreq = i + cfreq;
                 if (spfreq < 0)
                     continue;
@@ -442,18 +447,18 @@ void PADnoteParameters::generatespectrum_bandwidthMode(float *spectrum,
         else
         {   // if the bandwidth is smaller than the profilesize
             float rap = sqrtf((float)ibw / (float)profilesize);
-            float ibasefreq = realfreq / (samplerate * 0.5) * size;
+            float ibasefreq = realfreq / (synth->halfsamplerate_f) * size;
             for (int i = 0; i < profilesize; ++i)
             {
-                float idfreq = i / (float)profilesize - 0.5;
+                float idfreq = i / (float)profilesize - 0.5f;
                 idfreq *= ibw;
                 int spfreq = (int)(idfreq + ibasefreq);
-                float fspfreq = fmodf(idfreq + ibasefreq, (float)1.0);
+                float fspfreq = fmodf(idfreq + ibasefreq, 1.0f);
                 if (spfreq <= 0)
                     continue;
                 if (spfreq >= size - 1)
                     break;
-                spectrum[spfreq] += amp * profile[i] * rap * (1.0 - fspfreq);
+                spectrum[spfreq] += amp * profile[i] * rap * (1.0f - fspfreq);
                 spectrum[spfreq + 1] += amp * profile[i] * rap * fspfreq;
             }
         }
@@ -472,40 +477,40 @@ void PADnoteParameters::generatespectrum_otherModes(float *spectrum,
     //    spectrum[i] = 0.0;
     memset(spectrum, 0, sizeof(float) * size);
 
-    float harmonics[half_oscilsize];
-    memset(harmonics, 0, half_oscilsize * sizeof(float));
+    float harmonics[synth->halfoscilsize];
+    memset(harmonics, 0, synth->halfoscilsize * sizeof(float));
 
     // get the harmonic structure from the oscillator (I am using the frequency
     // amplitudes, only)
     oscilgen->get(harmonics, basefreq, false);
 
     // normalize
-    float max = 0.0;
-    for (int i = 0; i < half_oscilsize; ++i)
+    float max = 0.0f;
+    for (int i = 0; i < synth->halfoscilsize; ++i)
         if (harmonics[i] > max)
             max = harmonics[i];
-    if (max < 0.000001)
+    if (max < 0.000001f)
         max = 1;
-    for (int i = 0; i < half_oscilsize; ++i)
+    for (int i = 0; i < synth->halfoscilsize; ++i)
         harmonics[i] /= max;
 
-    for (int nh = 1; nh < half_oscilsize; ++nh)
+    for (int nh = 1; nh < synth->halfoscilsize; ++nh)
     {   //for each harmonic
         float realfreq = getNhr(nh) * basefreq;
 
         ///sa fac aici interpolarea si sa am grija daca frecv descresc
 
-        if (realfreq > samplerate * 0.49999)
+        if (realfreq > synth->samplerate_f * 0.49999f)
             break;
-        if (realfreq < 20.0)
+        if (realfreq < 20.0f)
             break;
 //	if (harmonics[nh-1]<1e-4) continue;
 
         float amp = harmonics[nh - 1];
         if (resonance->Penabled)
             amp *= resonance->getfreqresponse(realfreq);
-        int cfreq = (int)(realfreq / (samplerate * 0.5) * size);
-        spectrum[cfreq] = amp + 1e-9;
+        int cfreq = lrintf(realfreq / (synth->halfsamplerate_f) * size);
+        spectrum[cfreq] = amp + 1e-9f;
     }
 
     if (Pmode != 1)
@@ -513,16 +518,16 @@ void PADnoteParameters::generatespectrum_otherModes(float *spectrum,
         int old = 0;
         for (int k = 1; k < size; ++k)
         {
-            if ((spectrum[k] > 1e-10) || (k == (size - 1)))
+            if ((spectrum[k] > 1e-10f) || (k == (size - 1)))
             {
                 int delta = k - old;
                 float val1 = spectrum[old];
                 float val2 = spectrum[k];
-                float idelta = 1.0 / delta;
+                float idelta = 1.0f / delta;
                 for (int i = 0; i < delta; ++i)
                 {
                     float x = idelta * i;
-                    spectrum[old+i] = val1 * (1.0 - x) + val2 * x;
+                    spectrum[old+i] = val1 * (1.0f - x) + val2 * x;
                 }
                 old = k;
             }
@@ -541,7 +546,7 @@ void PADnoteParameters::applyparameters(bool islocked)
 
     float bwadjust = getprofile(profile, profilesize);
 //    for (int i=0;i<profilesize;i++) profile[i]*=profile[i];
-    float basefreq = 65.406 * powf(2.0f, Pquality.basenote / 2);
+    float basefreq = 65.406f * powf(2.0f, Pquality.basenote / 2);
     if (Pquality.basenote %2 == 1)
         basefreq *= 1.5;
 
@@ -565,10 +570,10 @@ void PADnoteParameters::applyparameters(bool islocked)
 
     float adj[samplemax]; // this is used to compute frequency relation to the base frequency
     for (int nsample = 0; nsample < samplemax; ++nsample)
-        adj[nsample] = (Pquality.oct + 1.0) * (float)nsample / samplemax;
+        adj[nsample] = (Pquality.oct + 1.0f) * (float)nsample / samplemax;
     for (int nsample = 0; nsample < samplemax; ++nsample)
     {
-        float tmp = adj[nsample] - adj[samplemax - 1] * 0.5;
+        float tmp = adj[nsample] - adj[samplemax - 1] * 0.5f;
         float basefreqadjust = powf(2.0f, tmp);
 
         if (Pmode == 0)
@@ -587,11 +592,11 @@ void PADnoteParameters::applyparameters(bool islocked)
         newsample.smp[0] = 0.0;
         for (int i = 1; i < spectrumsize; ++i)
         {   // randomize the phases
-            float phase = zynMaster->numRandom() * 6.29;
+            float phase = synth->numRandom() * 6.29f;
             fftfreqs.c[i] = spectrum[i] * cosf(phase);
             fftfreqs.s[i] = spectrum[i] * sinf(phase);
         }
-        fft->freqs2smps(fftfreqs, newsample.smp);
+        fft->freqs2smps(&fftfreqs, newsample.smp);
         // that's all; here is the only ifft for the whole sample; no windows are used ;-)
 
         // normalize(rms)
@@ -603,7 +608,7 @@ void PADnoteParameters::applyparameters(bool islocked)
             rms = 1.0;
         rms *= sqrtf(262144.0f / samplesize);
         for (int i = 0; i < samplesize; ++i)
-            newsample.smp[i] *= 1.0 / rms * 50.0;
+            newsample.smp[i] *= 1.0f / rms * 50.0f;
 
         // prepare extra samples used by the linear or cubic interpolation
         for (int i = 0; i < extra_samples; ++i)
@@ -611,13 +616,13 @@ void PADnoteParameters::applyparameters(bool islocked)
 
         // replace the current sample with the new computed sample
         if (!islocked)
-            zynMaster->actionLock(lockmute);
+            synth->actionLock(lockmute);
         deletesample(nsample);
         sample[nsample].smp = newsample.smp;
         sample[nsample].size = samplesize;
         sample[nsample].basefreq = basefreq * basefreqadjust;
         if (!islocked)
-            zynMaster->actionLock(unlock);
+            synth->actionLock(unlock);
         newsample.smp = NULL;
     }
     delete fft;
@@ -625,11 +630,11 @@ void PADnoteParameters::applyparameters(bool islocked)
 
     // delete the additional samples that might exists and are not useful
     if (!islocked)
-        zynMaster->actionLock(lockmute);
+        synth->actionLock(lockmute);
     for (int i = samplemax; i < PAD_MAX_SAMPLES; ++i)
         deletesample(i);
     if (!islocked)
-        zynMaster->actionLock(unlock);
+        synth->actionLock(unlock);
 }
 
 
